# TMF8701 MCU / Userspace driver

This is a Userspace / MCU driver for the Time-of-Flight TMF8701 sensor from
ams AG. It is meant to serve as a reference driver using (mostly) the standard
C library without dependencies on an external framework. The driver portions of
the example code are documented with Doxygen.

## Generating Doxygen from example code

*doxygen / dot* packages are required to build documentation

    1. mkdir build
    2. cd build
    3. cmake ..
    4. make docs
        - build all doxygen documentation
    6. make test_coverage
        - doxygen generated docs available at ./build/docs/*

## Building TMF8701 example application for Raspberry Pi Userspace

    1. mkdir build
    2. cd build
    3. cmake -DCMAKE_TOOLCHAIN_FILE=../toolchain_raspberrypi3.cmake ..
        - environment variable *RPI_TOOLCHAIN* must be set to the raspberry
          pi toolchain path
    4. make
        - output application is ./build/bin/mcu_tmf8701*

## Running the Unit Test suite

*gtest* package (headers + libs) is required to build unit test suite
*gcovr / lcov* packages are required for code coverage on unit test suite

    1. mkdir build
    2. cd build
    3. cmake ..
    4. make
        - build all test executables
    5. make test
        - run all unit test executables (must build first)
    6. make test_coverage
        - build & run unit test executables and collect code coverage data
            - GCOV data available at ./build/test/CMakeFiles/*
            - LCOV html available at ./build/test/test_coverage/*

## Porting the TMF8701 userspace driver to other platforms

- overwrite gpio interface for **ams_linux_gpio.h** / **ams_linux_gpio.c**
for target platform.
- overwrite i2c interface for **ams_linux_i2c.h** / **ams_linux_i2c.c**
for target platform.
- implement **_clock_gettime()_** for driver to have access to a
microsecond counter (used for clock correction and clock trimming)
- The **_rt_** library can be un-linked once clock_gettime() is implemented
- follow example **main.c** program for how to use interfaces for simple
continuous distance measurements with default settings

## Compiling the driver with the firmware built-in

*srecord* package (**srec_cat** utility) is needed to build the driver with
the firmware included.

    1. mkdir build
    2. cd build
    3. cmake -DFW_PATH=<path_to_firmware_hex_file> -DCMAKE_TOOLCHAIN_FILE=../toolchain_raspberrypi3.cmake ..
        - environment variable *RPI_TOOLCHAIN* must be set to the raspberry
          pi toolchain path
        - binary image source files ./build/src/generated/tof_bin_image* are generated and compiled in with
          the source
    4. make
        - output application is ./build/bin/mcu_tmf8701*

