/*
*****************************************************************************
* Copyright by ams AG                                                       *
* All rights are reserved.                                                  *
*                                                                           *
* IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING     *
* THE SOFTWARE.                                                             *
*                                                                           *
* THIS SOFTWARE IS PROVIDED FOR USE ONLY IN CONJUNCTION WITH AMS PRODUCTS.  *
* USE OF THE SOFTWARE IN CONJUNCTION WITH NON-AMS-PRODUCTS IS EXPLICITLY    *
* EXCLUDED.                                                                 *
*                                                                           *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  *
* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     *
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     *
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     *
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE     *
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      *
*****************************************************************************
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "ams_linux_gpio.h"

static char *sysfs_export = "/sys/class/gpio/export";
static char *sysfs_unexport = "/sys/class/gpio/unexport";
static char *sysfs_gpio = "/sys/class/gpio/gpio";

int32_t init_gpio(uint32_t gpio, uint32_t direction)
{
    int32_t error = 0;
    char cmd[255] = {0};

    if (gpio == 0 || direction >= INVAL_DIR)
        return -1;

    if (!gpio_is_init(gpio)) {
        (void) snprintf(cmd, sizeof(cmd), "echo %u > %s", gpio, sysfs_export);

        error = system(cmd);
        if (error)
            return error;
    }

    (void) snprintf(cmd, sizeof(cmd), "echo %s > %s%u/direction", direction ? "out" : "in", sysfs_gpio, gpio);

    error = system(cmd);

    return error;
}


bool gpio_is_init(uint32_t gpio)
{
    int32_t error = 0;
    char cmd[255] = {0};

    if (gpio == 0)
        return false;

    (void) snprintf(cmd, sizeof(cmd), "ls %s%u/value > /dev/null 2>&1", sysfs_gpio, gpio);

    error = system(cmd);
    return error ? false : true;
}

int32_t deinit_gpio(uint32_t gpio)
{
    int32_t error = 0;
    char cmd[255] = {0};

    if (gpio == 0 || !gpio_is_init(gpio))
        return -1;

    (void) snprintf(cmd, sizeof(cmd), "echo %u > %s", gpio, sysfs_unexport);

    error = system(cmd);
    return error;
}

int32_t read_gpio(uint32_t gpio, uint32_t *value)
{
    int32_t error = 0;
    char cmd[255] = {0};
    FILE *fp;

    if (!gpio_is_init(gpio) || value == NULL)
        return -1;

    (void) snprintf(cmd, sizeof(cmd), "cat %s%u/value", sysfs_gpio, gpio);

    fp = popen(cmd, "r");
    if (fp == NULL)
        return -1;

    error = fscanf(fp, "%u", value);
    (void)pclose(fp);

    return error;
}

int32_t write_gpio(uint32_t gpio, uint32_t value)
{
    int32_t error = 0;
    char cmd[255] = {0};

    if (!gpio_is_init(gpio))
        return -1;

    (void) snprintf(cmd, sizeof(cmd), "echo %u > %s%u/value", value, sysfs_gpio, gpio);

    error = system(cmd);
    return error;
}
