extern "C" {
#include "ams_linux_i2c.c"
}
#include <gtest/gtest.h>
#include <stdarg.h>
#include <fff.h>

FAKE_VALUE_FUNC_VARARG(int, ioctl, int, unsigned long, ...);

#define FFF_FAKES_LIST(FAKE)  \
    FAKE(ioctl)

class AmsLinuxI2cTest : public testing::Test
{
    void SetUp() {
        FFF_FAKES_LIST(RESET_FAKE);
        FFF_RESET_HISTORY();
    }
};

static uint8_t reg = {0};
static uint8_t buf[512] = {0};
static uint32_t len = 0;
static i2c_msg data[2];


int custom_write_fake(int fd, unsigned long req, va_list args)
{
    data[0].buf = buf;
    i2c_rdwr_ioctl_data *in = va_arg(args, i2c_rdwr_ioctl_data *);
    memset(buf, 0, sizeof(buf));
    memcpy(&data[0], in->msgs, sizeof(i2c_rdwr_ioctl_data));
    memcpy(buf, in->msgs[0].buf, in->msgs[0].len);
    len = in->msgs[0].len;
    return 0;
}

int custom_read_fake(int fd, unsigned long req, va_list args)
{
    i2c_rdwr_ioctl_data *in = va_arg(args, i2c_rdwr_ioctl_data *);
    memcpy(data, in->msgs, in->nmsgs*sizeof(i2c_rdwr_ioctl_data));
    printf("read reg: %#x\n", in->msgs[0].buf[0]);
    printf("read len: %#x\n", in->msgs[0].len);
    memcpy(&reg, in->msgs[0].buf, in->msgs[0].len);
    printf("reg: %#x\n", reg);
    memcpy(in->msgs[1].buf, buf, len);
    in->msgs[1].len = len;
    return 0;
}

TEST_F (AmsLinuxI2cTest, test_write)
{
    int rc = 0;
    uint8_t addr = 0xAA;
    uint8_t wreg = 0xBB;
    uint8_t rbuf[2] = {0xCC, 0xDD};
    int ioctl_rc[] = {-1, 0};
    SET_RETURN_SEQ(ioctl, ioctl_rc, 2);

    rc = write_i2c_block(addr, wreg, NULL, 0);
    ASSERT_EQ(rc, -1);

    char *tmp = i2cdev_fp;
    i2cdev_fp = NULL;
    rc = write_i2c_block(addr, wreg, rbuf, 2);
    ASSERT_EQ(rc, -1);
    i2cdev_fp = tmp;

    rc = write_i2c_block(addr, wreg, rbuf, 2);
    ASSERT_EQ(rc, -1);

    ioctl_fake.custom_fake = custom_write_fake;
    rc = write_i2c_block(addr, wreg, rbuf, 2);
    ASSERT_EQ(rc, 0);
    ASSERT_EQ(0, ioctl_fake.arg0_val);
    ASSERT_EQ(data[0].addr, addr);
    ASSERT_EQ(buf[0], wreg);
    ASSERT_EQ(buf[1], rbuf[0]);
    ASSERT_EQ(buf[2], rbuf[1]);
    ASSERT_EQ(len, sizeof(rbuf) + 1);
};

TEST_F (AmsLinuxI2cTest, test_read)
{
    int rc = 0;
    uint8_t addr = 0xAA;
    uint8_t rreg = 0xBB;
    uint8_t rbuf[2];
    buf[0] = 0xCC;
    buf[1] = 0xDD;
    len = 2;
    int ioctl_rc[] = {-1, 0};
    SET_RETURN_SEQ(ioctl, ioctl_rc, 2);

    rc = read_i2c_block(addr, rreg, NULL, 0);
    ASSERT_EQ(rc, -1);

    char *tmp = i2cdev_fp;
    i2cdev_fp = NULL;
    rc = read_i2c_block(addr, rreg, rbuf, 2);
    ASSERT_EQ(rc, -1);
    i2cdev_fp = tmp;

    rc = read_i2c_block(addr, rreg, rbuf, 2);
    ASSERT_EQ(rc, -1);

    ioctl_fake.custom_fake = custom_read_fake;
    rc = read_i2c_block(addr, rreg, rbuf, 2);
    ASSERT_EQ(rc, 0);
    ASSERT_EQ(0, ioctl_fake.arg0_val);
    ASSERT_EQ(data[0].addr, addr);
    ASSERT_EQ(data[1].addr, addr);
    ASSERT_EQ(reg, rreg);
    ASSERT_EQ(buf[0], rbuf[0]);
    ASSERT_EQ(buf[1], rbuf[1]);
};

